#!/usr/local/bin/php -c /usr/local/lib/php.ini

<?PHP
error_reporting(E_ERROR);
/*
* awstats installer.

* This works with the Direct Admin plugin to install awstats on a single 
* domain or all domains. If it installs awstats on a domain, it will add
* the user to permissions.txt unless -s is spaficied.
* 
* Parameters:
* -d not check permissions.txt -- false
* This is used in conjunction with any of the other parameters to cause the 
* program to ignore the fact that the domain being installed for is not in 
* permissions.txt
*
* -s skip updating permissions.txt -- false
* unless told to do so otherwise, the program will always update 
* permissions.txt if the user does not already exist. If you want to install 
* but NOT update permissions.txt, use this switch.
*
* -a all domains -- false
* Builds a list of all domains on the server and verifies that awstats in 
* installed properly on each domain. If the domain already has awstats 
* installed then nothing is done.
*
* -f force -- false
* Forces awstats to be installed over the top of an existing install.
*
* -d domain.tld -- unset
* Installs awstats on a single domain.
*
* -t test -- false
* Ok, sue me, I've already used -d for debug so I'll go with -t for test.  
* This will output stuff that is only of interest to developers.
*
* -h help -- false
* Generates a help message for the program.
*
*
* @example
* ./awstatsInstaller -v 
* Lists all the domains on the server. Nothing else.
*
* ./awstatsInstaller -a 
* Install or check integrity of awstats on all domains on the server. If awstats
* does not exist on a domain, it will install it. If it does exist, it will 
* check the pieces to make sure they are all correct.
*
* ./awstatsInstaller -d mydomain.com 
* Installs or checks awstats on the domain mydomain.com
*
* ./awstatsInstaller -fnd mydomain.com 
* Reinstalls awstats on mydomain.com regardless of whether it was installed 
* or not. This WILL overwrite the conf file and the .htaccess file. Skips 
* checking the permissions.txt file to see if this user has permission to run
* aswtats.
*
* ./awstatsInstaller -sd mydomain.com 
* Installs or checks awstats on the domain mydomain.com. Skips adding the 
* owner of mydomain.com to permissions.txt 
*
* ./awstatsInstaller -nd mydomain.com 
* Installs or checks awstats on the domain mydomain.com. If installing, ignores 
* whether or not the user is in permissions.txt. Use this flag when you have
* domains you want installed but have not updated your permissisons.txt. 
* Unless you ALSO specify -s, your permissions.txt will be updated.
*
* @author Cal Evans <cal@calevans.com>
* @copyright (c) 2005 Cal Evans All Rights Reserved
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
* @version 1.2
* @access public 
*
* @todo At some point I need to extract the stuff that is specific to
* DirectAdmin into a DABase class.
*
*
* Version Hitory:
* 1.0 01/03/2005 - Cal
* Initial Release
*
* 1.1 01/03/2005 - Cal
* 15 minutes later, the first patch. There was a problem with createSymLink. 
* It wouldn't create the proper links.
*
* 1.2 01/04/2005 - Cal
* In _processSingleDomain, changed the logic so that if awstats already 
* exists, it now doesn't just bail, it checks the site for integrity. This way 
* the system can be used for repair as well as install. (That is unless -f was 
* set. If -f then it WILL force a reinstall.)
*
* Added help switch.
*
* Added calling examples.
* 
*/


$o = & new AWStatsInstaller();
/*
* Parse the parameters that were passed in.
*/
$o->setOptions(getopt('afd:svth'));
/*
* FIRE!
*/
$o->main();

/*
* Forget
*/
$o=null;
die();



/**
* Manages installs of the awstats program for the awstats plugin.
*
* This program works in conjunction with the plugin for DirectAdmin developed 
* by fusionictnl. It allows for the installing or reinstalling of awstats on 
* a single domain to all the domains on a server. Mega Props go out to 
* fusionictnl. He wrote most of the logic, I just shoved it in an object 
* casing.
*
* @author Cal Evans <cal@calevans.com>
* @copyright (c) 2005 Cal Evans ALl Rights Reserved
* @version 1.0
* @access public 
*/
class AWStatsInstaller
{
	/**
	* the options array. Thisholds the options passed into the program.
	* @var array $_options
	* @access private
	*/
	var $_options;

	/**
	* flag for force mode
	* @var bool $_force
	* @access private
	*/
	var $_force;

	/**
	* flag for skipping the writing of a new permissions.txt file.
	* @var bool $_skipPerms
	* @access private
	*/
	var $_skipPerms;
	
	/**
	* flag for ignoring the permissions.txt file.
	* @var bool $_ignorePerms
	* @access private
	*/
	var $_ignorePerms;

	/**
	* flag for spewing forth to the console a complete audit of what is done.
	* @var bool $_verbose
	* @access private
	*/
	var $_verbose;

	/**
	* flag for spewing forth to the console usless debugging messages.
	* @var bool $_testMode
	* @access private
	*/
	var $_testMode;

	/**
	* if set and contains a domain name, awstats will be installed for this
	* domain only.
	* @var string  $_singleDomain
	* @access private
	*/
	var $_singleDomain;

	/**
	* flag for installing awstats on all domains on the server.
	* @var bool $_allDomains
	* @access private
	*/	
	var $_allDomains;

	/**
	* Array of errors that occured during execution.
	* @var array $_errors
	* @access private
	*/	
	var $_errors;

	/**
	* The full path, sans trailing / to the home dir.
	* @var string $_homeDir
	* @access private
	*/		
	var $_homeDir;
	

	/**
	* The full path, sans the trailing / for the DirectAdmin directory.
	* @var bool $_DADir
	* @access private
	*/	
	var $_DADir;

	/**
	* The pattern to use when looking for domains.
	* @var string $_domainListPattern
	* @access private
	*/		
	var $_domainListPattern;

	/**
	* Array of arrays. The contains one array for each domain. The key is the 
	* domain name. Each subarray contains 
	* - domain
	* - user
	* - path
	* @var array $_domains
	* @access private
	*/		
	var $_domains;

	/**
	* The full path, sans the trailing /, for the DirectAdmin plugin directory.
	* @var bool $_allDomains
	* @access private
	*/		
	var $_pluginDir;	

	
	/**
	* Array to store the contents of the permissions.txt file.
	* @var array $_permissions
	* @access private
	*/		
	var $_permissions;	

	/**
	* Flag to indicate the user has requested the help screen be displayed. 
	* This overrides ALL other flags. If this is true, nothing else will 
	* happen.
	* @var bool $_helpFlag
	* @access private
	*/		
	var $_helpFlag;	
	
	/**
	* constructor
	*
	* Sets reasonable defaults for all properties.
	*/
	function AWStatsInstaller()
	{
		$this->_options           = array();
		$this->_force             = false;
		$this->_skipPerms         = false;
		$this->_ignorePerms       = false;
		$this->_verbose           = false;
		$this->_testMode          = false;
		$this->_allDomains        = false;
		$this->_singleDomain      = null;
		$this->_errors            = array();
		$this->_homeDir           = '/home';
		$this->_DADir             = '/usr/local/directadmin';
		$this->_hooksDir          = $this->_DADir."/plugins/awstats/hooks";

		$this->_domains           = array();
		$this->_domainListPattern = '/*/domains/*/public_html';
		$this->_permissions       = array();
		$this->_helpFlag          = false;
	} // function AWStatsInstaller()
	
	
	/**
	* sets the options from the argv array.
	*
	* This should be the first call after creating the object. It takes the argv array passed in and sets the flags based on the options specified.
	* @param array $optionArray The argv array.
	* @returns bool always true
	*/
	function setOptions($optionArray)
	{
		
		$this->_allDomains   = isset($optionArray['a']);
		$this->_force        = isset($optionArray['f']);
		$this->_skipPerms    = isset($optionArray['s']);
		$this->_ignorePerms  = isset($optionArray['d']);
		$this->_verbose      = isset($optionArray['v']);
		$this->_testMode     = isset($optionArray['t']);
		$this->_helpFlag     = isset($optionArray['h']);
		if (!$this->_allDomains)
		{
			$this->_singleDomain = isset($optionArray['d'])?trim($optionArray['d']):null;
		}
		else 
		{
			if (!empty($optionArray['d'])) $this->_setError(1,'the "All Domains" flag overrides the single domain setting. All domains will be processed.');
		} // if (!$this->_allDomains)
		return true;
	} // function setOptions(&$optionArray)

	
	/**
	* Main executable body.
	*
	* This is the main function call. Once all options have been set, call this to execute.
	* @returns bool always true
	*/
	function main()
	{
		/*
		* If the user has requested help, show the help and bail. Yes, I know 
		* it's bad form to have more than one return in a method but in this 
		* case it beats wrapping the entire thing in another IF statement. 
		* (sue me!)
		*/
		if ($this->_helpFlag) 
		{
			$this->showHelp();
			return true;
		} // if ($this->_helpFlag) 
		
		/*
		* Reset these her in case something has changed.
		*/
		$this->_domainListPattern = $this->_homeDir.'/*/domains/*/public_html';
		$this->_hooksDir          = $this->_DADir."/plugins/awstats/hooks";
		$this->_readPermissisons();
		$this->_getDomainList();
		/*
		* I could do an either/or here but I'm not. This way, if I set -a to 
		* false and -s to null and -v then I'll get a printout of all the 
		* domains.
		*/
		if ($this->_allDomains) 
		{
			$this->_processAllDomains();
			if (!$this->_skipPerms) $this->_writePermissisons();
		} // if ($this->_allDomains) 
		if (!empty($this->_singleDomain)) 
		{
			/*
			* We'll make an assumption here. If the user has passed in a 
			* single domain then we'll asuume that they want them added to 
			* the permissions file. Otherwise it wouldn't make much sense.
			*/
			$this->_addPermissison($this->_domains[$this->_singleDomain]['user']);
			$returnValue = $this->_processSingleDomain($this->_domains[$this->_singleDomain]);
			if ($returnValue and !$this->_skipPerms) $this->_writePermissisons();
		}
		return true;	
	} // function main()
	
	
	/**
	* Gather a list of domains.
	*
	* This function uses the domainListPattern to build a list of all domains on th eserver. Regardless of whether or not we are processing a single domain, this function is always called.
	* @returns bool always true
	*/
	function _getDomainList()
	{
		if ($this->_testMode) echo "ls ".$this->_domainListPattern." | grep public_html | tr -d '\n'";		
		$this->storeMessage("Gathering domains");
		$list=exec("ls ".$this->_domainListPattern." | grep public_html | tr -d '\n'");
		$paths=split(":",$list);
		for ($lcvA=0;$lcvA<sizeof($paths)-1;$lcvA++) 
		{			
			list($nul,$nul,$user,$nul,$domain,$nul)=explode("/",$paths[$lcvA]);
			$this->storeMessage("Found domain: ".$domain);
			$this->_domains[$domain] = array('domain'=>$domain,'user'=>$user,'path'=>$paths[$lcvA]);
		} // for ($lcvA=0;$lcvA<sizeof($paths)-1;$lcvA++) 
		return true;
	} // function _getDomainList()
	
	
	/**
	* Read the permissions.txt file.
	*
	* Read the permissions.txt file and put it's contents in the permissions array.
	* @returns bool always true
	*/
	function _readPermissisons()
	{
		$this->_permissions = array();
		$this->_permissions = file($this->_DADir.'/plugins/awstats/hooks/permissions.txt');
		for($lcvA=0;$lcvA<count($this->_permissions);$lcvA++)
		{
			$this->_permissions[$lcvA]=str_replace("\n",'',$this->_permissions[$lcvA]);
		} // for($lcvA=0;$lcvA<count($this->_permissions);$lcvA++)
		return true;
	} // function _readPermissisons()
	

	/**
	* Check a user against the permissison table.
	*
	* @param string $targetUser The user to check for.
	* @returns bool true if the user exists in the array.
	*/
	function _checkPermissisons($targetUser='')
	{
		return (in_array($targetUser, $this->_permissions));
	} // function _checkPermissisons($targetDomain='')
	
	
	/**
	* Add a permission into the table.
	*
	* @param string $user The user to checkadd in.
	* @returns bool true if the user exists in the array.
	*/
	function _addPermissison($user='')
	{
		$returnValue = true;
		if (!empty($user) and !$this->_checkPermissisons($user))
		{
			$this->_permissions[]=$user;
		}
		else 
		{
			$returnValue = false;
		}
		return $returnValue;
	} // function _addPermissison($user='')
	
	
	/**
	* Write the permissions array out to permission.txt
	*
	* @returns bool always true;
	*/
	function _writePermissisons()
	{
		$outHand = fopen($this->_DADir.'/plugins/awstats/hooks/permissions.txt','w+');
		for($lcvA=0;$lcvA<count($this->_permissions);$lcvA++) fwrite($outHand,$this->_permissions[$lcvA]."\n");
		fClose($outHand);
		return true;
	} // function _writePermissisons()
	
	
	/**
	* places an error message in the errors array.
	* 
	* @param int errorNo The error number to report.
	* @param string errorMsg The error message to report.
	* @returns bool always true;
	*/
	function _setError($errorNo,$errorMsg)
	{
		$this->_errors[]=$errorNo.';'.$errorMsg;
		$this->storeMessage($errorNo.';'.$errorMsg);
		return true;
	} // function _setError($errorNo,$errorMsg)
	
	
	/**
	* Spins through the domains array and processes each one individually.
	*
	* At this point the variable $thisDomain is handing in an array, not a string. Also, we add the user to the permissions file here. If you don't want the permissions file updated then use the -s flag.
	*
	* @returns bool always true;
	*/
	function _processAllDomains()
	{
		foreach($this->_domains as $thisDomain) {
		if ($this->_testMode) echo "Processing ".$thisDomain['domain']."\n\n";
		if ($this->_testMode) echo "For user ".$thisDomain['user']."\n\n";
	        if ($this->_ignorePerms or $this->_checkPermissisons($thisDomain['user'])) { 
			if ($this->_testMode) echo "Pemissions installing ".$thisDomain['domain']."\n\n";
			$this->_addPermissison($thisDomain['user']);
			$this->_processSingleDomain($thisDomain);
		}
		} // foreach($this->_domains as $thisDomain) 
		return true;
	} // function _processAllDomains()
	
	
	/**
	* processes a single domain array.
	*
	* This is where all the work is done.
	* @param array domainToProcess the array containing the information necessary to process this domain.
	* @returns bool always true;
	*/
	function _processSingleDomain($domainToProcess=array())
	{
		$returnValue = true;
		if ($this->_testMode) echo "Processing ".$domainToProcess['domain']."\n\n";
		/*
		* Don't do anything if the array is empty.
		*/
		if (count($domainToProcess)>0)
		{
			/*
			* If the user is not in the permissions table and the ignore perms 
			* switch is not set then do nothing.
			*/
			if ($this->_ignorePerms or $this->_checkPermissisons($domainToProcess['user']))
			{
				$location=$domainToProcess['path'].'/awstats';
				if ($this->_testMode) echo "Location : ".$location;
				$this->storeMessage("Working on domain: ".$domainToProcess['domain']);
				$this->storeMessage("Checking for the awstats directory.");
				if ($this->_testMode) $this->storeMessage($location);
				if (!file_exists($location) or $this->_force)
				{
					@mkdir($location);
				}
				else 
				{
					$this->storeMessage("awstats already exists on this domain.");
					$this->storeMessage("We will continue and check the integrity of the install.");
				}

				$this->storeMessage("Setting Directory Permissions");
				chown($location,$domainToProcess['user']);
				chgrp($location,$domainToProcess['user']);
				
				$this->storeMessage("Checking if awstats files are available.");
				
				$this->_createSymLink($location.'/lib',     $this->_hooksDir."/cgi-bin/lib",     $this->_force);
				$this->_createSymLink($location.'/icon',    $this->_hooksDir."/cgi-bin/icon",    $this->_force);
				$this->_createSymLink($location.'/lang',    $this->_hooksDir."/cgi-bin/lang",    $this->_force);
				$this->_createSymLink($location.'/plugins', $this->_hooksDir."/cgi-bin/plugins", $this->_force);
				
				$this->storeMessage("Checking for awstats itself.");
				if (!file_exists($location.'/awstats.pl') or $this->_force)
				{
					$this->storeMessage((file_exists($location.'/awstats.pl')?'Re-':'')."Installing awstats.pl");
					copy ($this->_hooksDir.'/cgi-bin/awstats.pl',$location.'/awstats.pl');
					chmod ($location.'/awstats.pl',0755);
					chown ("$location/awstats.pl",$domainToProcess['user']);
					chgrp ("$location/awstats.pl",$domainToProcess['user']);
				} // if (!file_exists($location.'/.awstats.pl') or $this->_force)
				
				
				$this->storeMessage("Checking for .htaccess.");
				if (!file_exists($location.'/.htaccess') or $this->_force)
				{
					$this->storeMessage((file_exists($location.'/.htaccess')?'Re-':'')."Activating CGI for directory");
					if (file_exists($location.'/.htaccess')) unlink ($location.'/.htaccess');
					copy ("$this->_hooksDir/cgi-bin/.htaccess","$location/.htaccess");
					chmod ("$location/.htaccess",0755);
					chown ("$location/.htaccess",$domainToProcess['user']);
					chgrp ("$location/.htaccess",$domainToProcess['user']);
				} // if (!file_exists($location.'/.htaccess') or $this->_force)

				
				$this->storeMessage("Checking for the conf file.");
				if (!file_exists("/etc/awstats/awstats.{$domainToProcess['domain']}.conf") or 
				    $this->_force)
				{
					$blankFileName = $this->_hooksDir.'/cgi-bin/awstats.none.conf';
					$newFileName   = '/etc/awstats/awstats.'.$domainToProcess['domain'].'.conf';
					$this->storeMessage((file_exists($newFileName )?'Re-':'')."Creating your configuration file.");
					$fp   = fopen($blankFileName,'r');
					$fp2  = fopen($newFileName,'w+');
					$cont =fread($fp,filesize($blankFileName));
					fclose($fp);
					$cont = ereg_replace("#%LOG%#","/var/log/httpd/domains/{$domainToProcess['domain']}.log",$cont);
					$cont = ereg_replace("#%DOMAIN%#","www.{$domainToProcess['domain']}",$cont);
					fwrite($fp2,$cont);
					fclose($fp2);
					chown ($newFileName,$domainToProcess['user']);
					chgrp ($newFileName,$domainToProcess['user']);
				} // if (!file_exists("/etc/awstats/awstats.{$domainToProcess['domain']}.conf") or 

				$this->storeMessage("Domain ".$domainToProcess['domain']." is ready!");
			}
			else
			{
				$this->storeMessage($domainToProcess['user']." is not in the permissions file and ignore permissison is not set Skipping.");
				$returnValue = false;
			} // if ($this->_ignorePerms or $this->_checkPermissisons($domainToProcess['domain']) 
		} // if (count($domainToProcess)>0)
		return $returnValue;
	} // function _processSingleDomain($domainToProcess)
	
	
	/**
	* creates a symlink.
	*
	* First it checks to see if the symlink exists. If not then it creates it. If it already exists and force mode is called then it will delete it and recreate it. Otheriwse, it won't do anything.
	*
	* @param string target The symlink target. (The NEW file/dir)
	* @param string source The symlink source. (The OLD file/dir)
	* @returns bool true if a link was created.;
	*/
	function _createSymLink($target=null, $source=null, $force=false)
	{
		$returnValue = true;
		
		if (!is_null($source) and !is_null($target))
		{
			if (file_exists($target) && !$force) 
			{
				$this->storeMessage($target." exists.");
			}
			else 
			{ 
				/*
				* If we get here and the file exists already then we are in 
				* force mode. Just for grins and giggles, let's remove the 
				* link and re-create it.
				*/
				if (file_exists($target)) unlink($target);
				symlink($source,$target); 
				
				$this->storeMessage($target." created.");
			} // if (file_exists($target) or $force) 
		}
		else
		{
			$returnValue = false;
		} // if (!is_null($source) and !is_null($target))
		
		return $returnValue;
	} // function createSymLink($target=null, $source=null, $force=false)
	
	
	/**
	* stores system messages.
	*
	* Stores the messages in an array for review after the processing is finished. If verbose mode is on, it will also echo them to the console.
	*
	* @param string message the message to store/display.
	* @returns bool always true
	*/
	function storeMessage($message='')
	{
		$this->_messages[] = date('m/d/Y h:i:s').':'.$message;
		if ($this->_verbose) echo $message."\n";
		return true;
	} // function storeMessage($message='')

	function showHelp()
	{
		$outString =  "awstatsInstaller.php v1.2\n";
		$outString .= "  Cal Evans <cal@calevans.com>\n";
		$outString .= "\n";
		$outString .= "Works with DirectAdmin and the awstats plugin to\n";
		$outString .= "properly install awstats on all domains, a single domain or repair an installation.\n\n\n";
		$outString .= "Parameters:\n";
		$outString .= "-d do not check permissions.txt -- false\n";
		$outString .= "This is used in conjunction with any of the other parameters to cause the\n"; 
		$outString .= "program to ignore the fact that the domain being installed for is not in\n"; 
		$outString .= "permissions.txt\n";
		$outString .= "\n";
		$outString .= "-s skip updating permissions.txt -- false\n";
		$outString .= "unless told to do so otherwise, the program will always update \n";
		$outString .= "permissions.txt if the user does not already exist. If you want to install \n";
		$outString .= "but NOT update permissions.txt, use this switch.\n";
		$outString .= "\n";
		$outString .= "-a all domains -- false\n";
		$outString .= "Builds a list of all domains on the server and verifies that awstats in \n";
		$outString .= "installed properly on each domain. If the domain already has awstats \n";
		$outString .= "installed then nothing is done.\n";
		$outString .= "\n";
		$outString .= "-f force -- false\n";
		$outString .= "Forces awstats to be installed over the top of an existing install.\n";
		$outString .= "\n";
		$outString .= "-d domain.tld -- unset\n";
		$outString .= "Installs awstats on a single domain.\n";
		$outString .= "\n";
		$outString .= "-t test -- false\n";
		$outString .= "Ok, sue me, I've already used -d for debug so I'll go with -t for test.  \n";
		$outString .= "This will output stuff that is only of interest to developers.\n";
		$outString .= "\n";
		$outString .= "-h help -- false\n";
		$outString .= "Displays a help message for the program.\n";
		$outString .= "\n";
		echo $outString;
		
		return true;
	} // function showHelp()
} // class AWStatsInstaller

?>
